

/*
## Cypress Image Sensor Routines for XXXX sensor (cyu3imagesnesor.c)
## ===========================
##
##  Copyright Cypress Semiconductor Corporation, 2013,
##  All Rights Reserved
##  UNPUBLISHED, LICENSED SOFTWARE.
##
##  CONFIDENTIAL AND PROPRIETARY INFORMATION
##  WHICH IS THE PROPERTY OF CYPRESS.
##
##  Use of this file is governed
##  by the license agreement included in the file
##
##     <install>/license/license.txt
##
##  where <install> is the Cypress software
##  installation root directory path.
##
## ===========================
*/


#include <cyu3error.h>
#include <cyu3i2c.h>
#include "cyu3imagesensor.h"
#include <cyu3utils.h>



/*Uncomment below line to enable UART debug messages during Sensor configuration*/
/*#define SENSOR_DEBUG*/



/* Misc Macros; Fill the correct I2C addresses below */
#define SENSOR_I2C_READ_ADDRESS         (0xFF) //Read address = 0xFF 
#define SENSOR_I2C_WRITE_ADDRESS        (0xFE) //Write address = 0xFE

/* Register Addresses */


/* Structure to store the Address/Value pair for each
 * register to be updated. */
typedef struct CySensorReg8
{
    uint16_t regAddr;
    uint8_t regValue;
} CySensorReg8_t;

typedef struct CySensorReg16
{
    uint16_t regAddr;
    uint16_t regValue;
} CySensorReg16_t;

typedef struct CySensorReg32
{
    uint16_t regAddr;
    uint32_t regValue;
} CySensorReg32_t;


/* Globals */


/* Configuration Settings*/


/* Configuration Settings for Sensor Init*/
CySensorReg16_t SENSOR_BaseConfigurationSettings [] =
{
/*
    {0xF000, 0x0101},
    {0xF008, 0x0202},
    {0xF008, 0x0202},
    {0xF103, 0x0303},
*/
	/* These are dummy Configuration Settings for Sensor. Replace with actual settings*/
};


					
/* Configuration settings for Resolution0 */
CySensorReg16_t SENSOR_Resolution0ConfigurationSettings [] =
{
/*
    {0xF000, 0x0101},
    {0xF008, 0x0202},
    {0xF008, 0x0202},
    {0xF103, 0x0303},
*/
	/* These are dummy Configuration Settings for Sensor. Replace with actual settings*/    
};


CyU3PReturnStatus_t I2C_SensorRead(uint16_t regAddr, uint8_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[3] = SENSOR_I2C_READ_ADDRESS; /* Slave address: Read operation */
        preamble.buffer[0] = SENSOR_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 4;
        preamble.ctrlMask = 0x0004;

        status = CyU3PI2cReceiveBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef SENSOR_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorRead Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}
CyU3PReturnStatus_t I2C_SensorWrite(uint16_t regAddr, uint16_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[0] = SENSOR_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 3;
        preamble.ctrlMask = 0x0000;

        status = CyU3PI2cTransmitBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef SENSOR_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorWrite Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}

/*Use the below function to set/clear fields of 16-bit register*/
void SENSOR_WR_FEILD(uint16_t RegAddr, uint16_t Field, CyBool_t FieldValue)
{
    uint8_t WRBuff[2];
    uint8_t RDBuff[2];
    uint16_t CamRegVal_16;

    I2C_SensorRead(RegAddr, 2, RDBuff);
    CamRegVal_16 = RDBuff[1];
    CamRegVal_16 |=RDBuff[0]<<8;

    if (FieldValue == 0)
    	CamRegVal_16 = (CamRegVal_16 & (~(Field)));
    else if (FieldValue == 1)
    	CamRegVal_16 = (CamRegVal_16 | (Field));

    WRBuff[0] = (CamRegVal_16 & 0xFF00)>>8;
    WRBuff[1] = (CamRegVal_16 & 0x00FF);
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint(4,"\r0x%x = 0x%x 0x%x\r\n",RegAddr,WRBuff[0], WRBuff[1]);
#endif

    I2C_SensorRead(RegAddr, 2, WRBuff);
}

/*Use the below function to write to 8-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg8 (CySensorReg8_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[2];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        
		rwBuffer[0] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 1, &rwBuffer); 
    }

    return status;
}

/*Use the below function to write to 16-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg16 (CySensorReg16_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[2];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer[0] = (((configSettings[regCounter].regValue)>>8)&0xFF);
		rwBuffer[1] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 2, &rwBuffer); 
    }

    return status;
}

/*Use the below function to write to 32-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg32 (CySensorReg32_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[4];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer[0] = (((configSettings[regCounter].regValue)>>24)&0xFF);
        rwBuffer[1] = (((configSettings[regCounter].regValue)>>16)&0xFF);
        rwBuffer[2] = (((configSettings[regCounter].regValue)>>8)&0xFF);
		rwBuffer[3] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 4, &rwBuffer); 
    }

    return status;
}

					
/* Configure Sensor for Resolution0*/

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Resolution0()
{
    uint16_t configSize;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    configSize = (sizeof(SENSOR_Resolution0ConfigurationSettings))/(sizeof(CySensorReg16_t));
    status =  SENSOR_WriteConfigurationSettings_Reg16(SENSOR_Resolution0ConfigurationSettings, configSize);
    
    return status;
}



/*Use the below function to initialize Image Sensor*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Init()
{
	uint16_t configSize;

#ifdef SENSOR_DEBUG
    CyU3PDebugPrint(4,"\r\n Sensor Initialization");
#endif

    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	configSize = (sizeof(SENSOR_BaseConfigurationSettings))/(sizeof(CySensorReg16_t));
    status = SENSOR_WriteConfigurationSettings_Reg16(SENSOR_BaseConfigurationSettings, configSize);
	
	/*Include any other I2C write required during Sensor Init here*/

    return status;
}

/*Use the below function to put Image Sensor to Sleep*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Sleep()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    return status;
}

/*Use the below function to wakeup Image Sensor*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Wakeup()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}


CyU3PReturnStatus_t CyCx3_ImageSensor_Trigger_Autofocus()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Format(CyU3PSensorStreamFormat_t format)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}


